/*++
Module Name:
    IO.cpp

Abstract:
    This file contains the queue entry points and callbacks.

Environment:
    Kernel-mode Driver Framework

--*/

#include "driver.h"
#include "IO.tmh"


UCHAR InBar8(const TBarData Bar, ULONG Offset)
{
    UCHAR Result;
    PUCHAR Addr = (PUCHAR)(Bar.Base + Offset);
    if (Bar.BarType == btMemory)
        Result = READ_REGISTER_UCHAR(Addr);
    else
        Result = READ_PORT_UCHAR(Addr)
        ;
    return Result;
}

#pragma LOCKEDCODE

USHORT InBar16(const TBarData Bar, ULONG Offset)
{
    USHORT Result;
    PUSHORT Addr = (PUSHORT)(Bar.Base + Offset);
    if (Bar.BarType == btMemory)
        Result = READ_REGISTER_USHORT(Addr);
    else
        Result = READ_PORT_USHORT(Addr)
        ;
    return Result;
}

#pragma LOCKEDCODE

ULONG InBar32(const TBarData Bar, ULONG Offset)
{
    ULONG Result;
    PULONG Addr = (PULONG)(Bar.Base + Offset);
    if (Bar.BarType == btMemory)
        Result = READ_REGISTER_ULONG(Addr);
    else
        Result = READ_PORT_ULONG(Addr)
        ;
    return Result;
}


#pragma LOCKEDCODE

void OutBar8(const TBarData Bar, ULONG Offset, UCHAR Data)
{
    PUCHAR Addr = (PUCHAR)(Bar.Base + Offset);
    if (Bar.BarType == btMemory)
        WRITE_REGISTER_UCHAR(Addr, Data);
    else
        WRITE_PORT_UCHAR(Addr, Data)
        ;
}

#pragma LOCKEDCODE

void OutBar16(const TBarData Bar, ULONG Offset, USHORT Data)
{
    PUSHORT Addr = (PUSHORT)(Bar.Base + Offset);
    if (Bar.BarType == btMemory)
        WRITE_REGISTER_USHORT(Addr, Data);
    else
        WRITE_PORT_USHORT(Addr, Data)
        ;
}

#pragma LOCKEDCODE

void OutBar32(const TBarData Bar, ULONG Offset, ULONG Data)
{
    PULONG Addr = (PULONG)(Bar.Base + Offset);
    if (Bar.BarType == btMemory)
        WRITE_REGISTER_ULONG(Addr, Data);
    else
        WRITE_PORT_ULONG(Addr, Data)
        ;
}


#pragma LOCKEDCODE

VOID InCOSData(PDEVICE_CONTEXT pdx, PUCHAR pDst, size_t COSSize)
{
    ULONG PPISize = pdx->COSScheme.Size;
    ULONG Offset = pdx->COSScheme.StartOff;
    ULONG dStride = pdx->COSScheme.Stride - pdx->COSScheme.Size;
    while (COSSize--)
    {
        if (PPISize == 0)
        {
            PPISize = pdx->COSScheme.Size;
            Offset += dStride;
        }
        --PPISize;

        *(pDst++) = InBar8(pdx->MainBar, Offset++);
    }
}
